
GO
PRINT N'Modificando [dbo].[ReportFieldAttribute]...';


GO
ALTER TABLE [dbo].[ReportFieldAttribute]
    ADD [IsMultiple] BIT DEFAULT ('0') NULL;


GO
PRINT N'Modificando [dbo].[GetReportProcedureParameters]...';


GO
ALTER PROCEDURE [dbo].[GetReportProcedureParameters](
	@Procedure Varchar(200)
	)
AS
BEGIN
	SET NOCOUNT ON

    DECLARE 
        @object_id INT,
        @paramID INT,
        @paramName SYSNAME,
        @definition NVARCHAR(MAX),
        @t NVARCHAR(MAX),
        @loc1 INT,
        @loc2 INT,
        @loc3 INT,
        @loc4 INT,
        @has_default_value BIT;

    SET @object_id = OBJECT_ID(@Procedure);

    IF (@object_id IS NOT NULL)
    BEGIN
    
        SELECT @definition = OBJECT_DEFINITION(@object_id);

        CREATE TABLE #params
        (
            parameter_id        INT PRIMARY KEY,
            has_default_value    BIT NOT NULL DEFAULT (0)
        );

        DECLARE c CURSOR
        LOCAL FORWARD_ONLY STATIC READ_ONLY
        FOR 
            SELECT 
                parameter_id,
                [name]
            FROM
                sys.parameters
            WHERE
                [object_id] = @object_id;

        OPEN c;

        FETCH NEXT FROM c INTO @paramID, @paramName;

        WHILE (@@FETCH_STATUS = 0)
        BEGIN
    
            SELECT
                @t = SUBSTRING
                (
                    @definition,
                    CHARINDEX(@paramName, @definition),
                    4000
                ),
                @has_default_value = 0;
            
            SET @loc1 = COALESCE(NULLIF(CHARINDEX('''', @t), 0), 4000);
            SET @loc2 = COALESCE(NULLIF(CHARINDEX(',', @t), 0), 4000);
            SET @loc3 = NULLIF(CHARINDEX('OUTPUT', @t), 0);
            SET @loc4 = NULLIF(CHARINDEX('AS', @t), 0);
            
            SET @loc1 = CASE WHEN @loc2 < @loc1 THEN @loc2 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc3 < @loc1 THEN @loc3 ELSE @loc1 END;
            SET @loc1 = CASE WHEN @loc4 < @loc1 THEN @loc4 ELSE @loc1 END;
            
            IF CHARINDEX('=', LTRIM(RTRIM(SUBSTRING(@t, 1, @loc1)))) > 0
                SET @has_default_value = 1;

            INSERT #params
            (
                parameter_id,
                has_default_value
            )
            SELECT
                @paramID,
                @has_default_value;
            
            FETCH NEXT FROM c INTO @paramID, @paramName;
        END
        
        SELECT 
            sp.[name]						'parameter',
            --sp.[parameter_id]				'ordinal',
            st.[name]						'type',
           --sp.[max_length]				'maxLength',
           --sp.[precision]					'precission',
           --sp.[scale]						'scale',
           --sp.[is_output]					'isOutput',
            p.[has_default_value]			'hasDefaultValue',
            IIF(RFA.[Label] IS NULL,REPLACE(sp.[name],'@',''),RFA.[Label])               'label',
            IIF(RFA.[IsDisabled] IS NULL,CAST(0 AS BIT),RFA.[IsDisabled])                'isDisabled',
            IIF(RFA.[IsDropdown] IS NULL,CAST(0 AS BIT),RFA.[IsDropdown])                'isDropdown',
            IIF(RFA.[IsMultiple] IS NULL,CAST(0 AS BIT),RFA.[IsMultiple])                'isMultiple',
            RFA.[Procedure]                 'procedure',
            RFA.[Arguments]                 '_arguments',
            RFA.[Variables]                 'variables'
        FROM
            sys.parameters sp
        INNER JOIN
            #params p
        ON
            sp.parameter_id = p.parameter_id
        INNER JOIN
            sys.types st
        ON
            sp.user_type_id = st.user_type_id
        LEFT JOIN 
                [dbo].[ReportFieldAttribute] RFA (NOLOCK)
         ON RFA.[Field] =  sp.[name]
        
        WHERE
            sp.[object_id] = @object_id;
        
        CLOSE c;
        DEALLOCATE c;
        DROP TABLE #params;
    END
END
GO
PRINT N'Modificando [dbo].[ReportByDetailAgent]...';


GO


ALTER PROCEDURE [dbo].[ReportByDetailAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL , 
	@Plan varchar(MAX) = NULL,
	@Format varchar(MAX), 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAuditor varchar(MAX) = NULL ,
	@IdAgente varchar(MAX) = NULL
) AS
  DECLARE @cols varchar(max) 
  DECLARE @query varchar(max)
  
  -- ReportByDetailAgent 'allegro','entrantes','Plan_2_nov01-0356','nov01-0356','20191001','20191201','isidoro','icagente3'
  SET NOCOUNT ON

  SELECT
    * INTO #TempEvaluationResult
  FROM (
  SELECT
	EP.[Instance]														'VCC',
    EP.Description														AS 'DescriptionPlan',
    EF.Description														AS 'DescriptionFormat',
    (
		GP.Description
		+ '-' 
		+ SGP.Description
		+ '-' 
		+ A.Title
	)																	AS 'Ability', --Collate SQL_Latin1_General_CP1253_CI_AI
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ISNULL(ER.Qualification, 0)											AS 'Qualification',
	AEP.[AutomaticScore],
	AEP.[ManualScore],
	--Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss')					AS 'EvaluationDate',
	--CAST(Format(AEP.[TimeStamp], 'yyyy-MM-dd hh:mm:ss') AS DATETIME)					AS 'EvaluationDate',
	AEP.[TimeStamp] AS 'EvaluationDate',
    ER.IdAuditor,
	EP.Evaluation_Mode	
  FROM EvaluationResult (NOLOCK) ER
  INNER JOIN EvaluationFormat EF
    ON ER.IdEvaluationFormat = EF.ID
  INNER JOIN Ability (NOLOCK) A
    ON A.[Instance] = ER.[Instance]
		AND A.ID = ER.IdAbility
  INNER JOIN EvaluationPlan (NOLOCK) EP
    ON EP.ID = ER.IdEvaluationPlan
	INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
		ON AEP.[Instance] = ER.[Instance]
			AND AEP.[IdEvaluationPlan] =  ER.[IdEvaluationPlan]
			AND AEP.[IdInteraction] = ER.[IdInteraction]
			AND AEP.[IdAuditor] =  ER.[IdAuditor]
  LEFT JOIN [Group] (NOLOCK) GP
    ON GP.ID = ER.IdGroup
  LEFT JOIN SubGroup (NOLOCK) SGP
    ON SGP.ID = ER.IdSubGroup
  WHERE 
	ER.Instance LIKE CASE
		WHEN @VCC IS NOT NULL THEN @VCC
		ELSE ER.Instance
	  END
  AND 
	EF.Description IN(SELECT [Value] FROM [dbo].[Split](IIF(@Format IS NULL,EF.Description,@Format),','))
  AND 
	EP.Description IN(SELECT [Value] FROM [dbo].[Split](IIF(@Plan IS NULL,EP.Description,@Plan),','))
  AND 
    ER.CampaignDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@HistoricCampaign IS NULL,ER.CampaignDescription,@HistoricCampaign),','))
  AND 
    ER.AgentDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAgente IS NULL,ER.AgentDescription,@IdAgente),','))
	
  AND 
    ER.IdAuditor IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAuditor IS NULL,ER.IdAuditor,@IdAuditor),','))

  AND AEP.[TimeStamp] >=
                          CASE
                            WHEN @StartDate IS NOT NULL THEN @StartDate
                            ELSE '19800101'
                          END
  AND AEP.[TimeStamp] <
                         CASE
                           WHEN @EndDate IS NOT NULL THEN @EndDate + ' 23:59'
                           ELSE '20500101'
                         END
  --AND EP.[Status] = 7
  AND AEP.[Completed] = 1
	) a

	--select * from #TempEvaluationResult ORDER BY IdInteraction


  SET @cols = STUFF((SELECT DISTINCT
    ',' + QUOTENAME(Ability) 
  FROM #TempEvaluationResult
  FOR xml PATH (''), TYPE)
  .value('.', 'varchar(Max)')
  , 1, 1, '')

  --Select @cols
  IF @cols IS NOT NULL
  BEGIN

    DECLARE @type varchar(max)=(Select Top 1 Evaluation_Mode From #TempEvaluationResult)

    DECLARE @Total varchar(max) = STUFF((SELECT DISTINCT
      '+ ISNULL(' + QUOTENAME(Ability) + ',0)'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')
    --SELECT @type
	If @type = 'M'
	BEGIN
    SET @query = ' 
				Select p.[VCC]							AS [VCC],
						p.DescriptionPlan				As [Plan],
						p.DescriptionFormat				As Formato,
						p.AgentDescription				As Agente,
  						p.CampaignDescription			As [Campana],
						p.IdInteraction					As [Interaccion],
						p.EvaluationDate				As [Fecha Evaluacion],
						p.IdAuditor						As Auditor,
						InteractionDuration				as [Tiempo duracion], 
						null							As [Telefono],
						InteractionDateStart			As [Fecha Inicio],
						null							As [Categoria],
						(' + @Total + ')				As COLUMN_TOTAL_SCORE, 
						p.[AutomaticScore]				AS COLUMN_AUTOMATIC_SCORE,
						p.[ManualScore]					AS COLUMN_MANUAL_SCORE,
						' + @cols + ' 
					from 
				( select VCC,
							DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  AutomaticScore,
						  ManualScore,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock) 
				) as A On
				A.IdInteraction = p.IdInteraction '
    END
	ELSE 
	BEGIN
    SET @query = ' 
				Select p.[VCC]							AS [VCC],
						p.DescriptionPlan				As [Plan],
						p.DescriptionFormat				As Formato,
						p.AgentDescription				As Agente,
  						p.CampaignDescription			As [Campana] ,
						p.IdInteraction					As [Interaccion] ,
						p.EvaluationDate				As [Fecha Evaluacion],
						p.IdAuditor						As Auditor,
						A.DurationTime					as [Tiempo Duracion],
						A.Number						As [Telefono] ,
						A.StartDate						As [Fecha Inicio] ,
						A.Category1						As Categoria ,
						(' + @Total + ')				As COLUMN_TOTAL_SCORE,
						p.[AutomaticScore]				AS COLUMN_AUTOMATIC_SCORE,
						p.[ManualScore]					AS COLUMN_MANUAL_SCORE,
						' + @cols + ' 
					from ( 
						select VCC,
								DescriptionPlan,
								DescriptionFormat,
								Ability,
								AgentDescription,
								CampaignDescription,
								IdInteraction,
								EvaluationDate,
								Qualification,
								AutomaticScore,
								ManualScore,
								IdAuditor			
							from #TempEvaluationResult
					) x pivot 
					( 
						 max(Qualification)
						for Ability in (' + @cols + ')
					) p 
					LEFT Join Audio as A 
						On A.[VCC] = p.[VCC]
							AND A.ID = p.IdInteraction'
	END
	--Print(@query)
    EXECUTE (@query)
    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    SELECT
      '' AS [Plan],
      '' AS Formato,
      '' AS Agente,
      '' AS [Campana],
      '' AS [Interaccion],
      '' AS EvaluationDate,
      '' AS IdAuditor
  END
  
  SET NOCOUNT OFF
GO
PRINT N'Modificando [dbo].[ReportBySummaryAgent]...';


GO


ALTER PROCEDURE [dbo].[ReportBySummaryAgent] (
	@VCC varchar(50) = NULL, 
	@HistoricCampaign varchar(MAX) = NULL, 
	@Plan varchar(MAX) = NULL,
	@Format varchar(MAX) , 
	@StartDate DateTime = NULL , 
	@EndDate DateTime = NULL , 
	@IdAuditor varchar(MAX) = NULL ,
	@IdAgente varchar(MAX) = NULL
) AS
  DECLARE @cols varchar(max)
  DECLARE @query varchar(max)
  --ReportBySummaryAgent 'allegro','entrantes',NULL,'nov01-0356',NULL,NULL,NULL,NULL
  --SET @Format = 'nov01-0356'
  SET NOCOUNT ON
  
  SELECT
    * INTO #TempEvaluationResult
  FROM (
  SELECT
    EP.Description AS 'DescriptionPlan',
    EF.Description AS 'DescriptionFormat',
    GP.Description + '-' + SGP.Description + '-' + A.Title AS 'Ability',
    ER.AgentDescription,
    ER.CampaignDescription,
    ER.IdInteraction,
    ER.Qualification,
    Format(ER.EvaluationDate, 'dd/MM/yyyy hh:mm:ss') AS 'EvaluationDate',
    ER.IdAuditor,
    EP.Evaluation_Mode
  FROM EvaluationResult (NOLOCK) ER
  LEFT JOIN EvaluationFormat (NOLOCK) EF
    ON ER.IdEvaluationFormat = EF.ID
  LEFT JOIN Ability (NOLOCK) A
    ON A.ID = ER.IdAbility
  LEFT JOIN EvaluationPlan (NOLOCK) EP
    ON EP.ID = ER.IdEvaluationPlan
  LEFT JOIN [Group] (NOLOCK) GP
    ON GP.ID = ER.IdGroup
  LEFT JOIN SubGroup SGP (NOLOCK)
    ON SGP.ID = ER.IdSubGroup
  WHERE 
   EF.Description IN(SELECT [Value] FROM [dbo].[Split](IIF(@Format IS NULL,EF.Description,@Format),','))
  AND ER.Instance LIKE CASE
    WHEN @VCC IS NOT NULL THEN @VCC
    ELSE ER.Instance
  END
  AND 
  EP.Description IN(SELECT [Value] FROM [dbo].[Split](IIF(@Plan IS NULL,EP.Description,@Plan),','))
  AND
  ER.CampaignDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@HistoricCampaign IS NULL,ER.CampaignDescription,@HistoricCampaign),','))
  AND 
  ER.AgentDescription IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAgente IS NULL,ER.AgentDescription,@IdAgente),','))
  AND 
  ER.IdAuditor IN(SELECT [Value] FROM [dbo].[Split](IIF(@IdAuditor IS NULL,ER.IdAuditor,@IdAuditor),','))
  AND ER.EvaluationDate >=
                          CASE
                            WHEN @StartDate IS NOT NULL THEN @StartDate
                            ELSE '19800101'
                          END
  AND ER.EvaluationDate <
                         CASE
                           WHEN @EndDate IS NOT NULL THEN @EndDate + ' 23:59'
                           ELSE '20500101'
                         END
	) a

	
  SET @cols = STUFF((SELECT DISTINCT
    ',' + QUOTENAME(Ability)
  FROM #TempEvaluationResult
  FOR xml PATH (''), TYPE)
  .value('.', 'varchar(Max)')
  , 1, 1, '')
  
  IF @cols IS NOT NULL
  BEGIN
    ------
    DECLARE @type varchar(max) = (SELECT TOP 1
      Evaluation_Mode
    FROM #TempEvaluationResult)
	-------
    DECLARE @Total varchar(max) = STUFF((SELECT DISTINCT
      '+ ISNULL(' + QUOTENAME(Ability) + ',0)'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')    
	-- ROUND(AVG(CAST(ER.Qualification AS float)), 2) AS 'Qualification',
    DECLARE @colavg varchar(max) = STUFF((SELECT DISTINCT
      ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
    FROM #TempEvaluationResult
    FOR xml PATH (''), TYPE)
    .value('.', 'varchar(Max)')
    , 1, 1, '')
	
	--SELECT @colavg
    IF @type = 'M'
    BEGIN
      SET @query = ' 
				Select q.DescriptionPlan As [Plan] ,q.DescriptionFormat As Formato ,q.AgentDescription As Agente ,q.CampaignDescription As Campana ,count(q.IdInteraction) As Auditorias,
				sum(q.DurationTime) As [Tiempo Duracion],ROUND(AVG(CAST(q.Note AS float)), 2) As [Nota Promedio], ' + @colavg + ' From
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,InteractionDuration as DurationTime, 
				null As Number,InteractionDateStart As StartDate,null As Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join (
				Select Distinct IdInteraction,InteractionDuration,InteractionDateStart From InteractionEvaluationPlan (Nolock)
				) as A On
				A.IdInteraction=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription
				'
    END
    ELSE
    BEGIN
      SET @query = '
	
			    Select q.DescriptionPlan As [Plan] ,q.DescriptionFormat As Formato ,q.AgentDescription As Agente,q.CampaignDescription As Campana,count(q.IdInteraction) As Auditorias,
				sum(q.DurationTime) As [Tiempo Duraccion],ROUND(AVG(CAST(q.Note AS float)), 2) As [Nota Promedio], ' + @colavg + ' From 
				(Select p.DescriptionPlan,p.DescriptionFormat,p.AgentDescription,
  				p.CampaignDescription,p.IdInteraction,p.EvaluationDate,p.IdAuditor,
				A.DurationTime,A.Number,A.StartDate,A.Category1,(' + @Total + ') As Note, ' + @cols + ' from 
				( select DescriptionPlan,
						  DescriptionFormat,
						  Ability,
						  AgentDescription,
						  CampaignDescription,
						  IdInteraction,
						  EvaluationDate,
						  Qualification,
						  IdAuditor			
					from #TempEvaluationResult
			   ) x
				pivot 
				( 
					 max(Qualification)
					for Ability in (' + @cols + ')
				) p 
				Left Join Audio as A On
				A.ID=p.IdInteraction)q
				Group By 
				q.DescriptionPlan,q.DescriptionFormat,q.AgentDescription,q.CampaignDescription'
    END
    --Print(@query)
    EXECUTE (@query)
    --Select * From @query
	

    DROP TABLE #TempEvaluationResult
  END
  ELSE
  BEGIN
    DROP TABLE #TempEvaluationResult
    SELECT
      '' AS [Plan],
      '' AS Formato,
      '' AS Agente,
      '' AS Campana
  END
  
  SET NOCOUNT OFF
GO
PRINT N'Actualización completada.';


GO
